<?php
/**
 * EMailer class file.
 * 
 * @category  Packages
 * @package   Ext.emailer
 * @author    MetaYii <undefined@example.com>
 * @copyright 2008 Copyright &copy; MetaYii
 * @license   http://www.gnu.org/licenses/lgpl.html LGPL
 * @link      http://www.yiiframework.com/phpmailer
 *
 *  Copyright (C) 2008 MetaYii.
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser General Public License as published by
 *     the Free Software Foundation, either version 2.1 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * For third party licenses and copyrights, please see:
 *
 * @see       phpmailer/LICENSE
 *
 */

/**
 * Include the the PHPMailer class.
 */
require_once dirname(__FILE__).'/phpmailer/class.phpmailer.php';

/**
 * EMailer is a simple wrapper for the PHPMailer library.
 *
 * @category Packages
 * @package  Ext.emailer
 * @author   MetaYii <undefined@example.com>
 * @license  http://www.gnu.org/licenses/lgpl.html LGPL
 * @link     http://www.yiiframework.com/phpmailer
 * @see      http://phpmailer.codeworxtech.com/index.php?pg=phpmailer
 * @since    1.1
 */
class EMailer extends CComponent
{
    /**
     * The internal PHPMailer object.
     * @var object PHPMailer
     */
    private $_myMailer;

    /**
     * Initializtion mode
     * @var string
     */
    public $init = 'IsSMTP';
    
    /**
     * ReplyTo address defaults
     * @var mixed
     */
    public $ReplyTo;

    /**
     * Startup initialization
     *
     * @return void
     */
    public function init()
    {
        if (!empty($this->ReplyTo)) {
            $this->_myMailer->AddReplyTo($this->ReplyTo);
        }
        $this->_myMailer->{$this->init}();
    }
    
    /**
    * Renders email's body and sets result content into $this->_myMailer->Body
    * 
    * @param string $view   alias to view for rendering email's content
    * @param array  $vars   array of view parameters used for rendering email's content
    * @param string $layout alias to layout view for rendering email's content
    *
    * @return void
    */
    public function getView($view, $vars = array(), $layout = null)
    {
        $app = Yii::app();
        $viewRenderer = $app->getComponent('viewRenderer');
        if ($app->controller instanceof CController) {
            $renderer = $app->controller;
            $baseArgs = array();
        } else {
            $renderer = $viewRenderer;
            $baseArgs = array($renderer);
        }
        $viewFile = $this->getViewPath($view);
        $renderMethod = array($renderer, 'renderFile');
        $params = array_merge($vars, array('mailer' => $this));
        $args = array_merge($baseArgs, array($viewFile, $params, true));
        $body = call_user_func_array($renderMethod, $args);
        if ($layout === null) {
            $this->setBody($body);
        } else {
            $layoutFile = $this->getViewPath($layout);
            $params = array('content' => $body, 'params' => $params);
            $args = array_merge($baseArgs, array($layoutFile, $params, true));
            $this->setBody(call_user_func_array($renderMethod, $args));
        }
    }
    
    /**
     * Changes email body
     * 
     * @param string $content new email body
     * 
     * @return void
     */
    public function setBody($content)
    {
        $this->_myMailer->Body = $content;
    }
    
    /**
     * Returns email body
     * 
     * @return string body content
     */
    public function getBody()
    {
        return $this->_myMailer->Body;
    }

    /**
     * Resolves absolute path to view file
     *
     * @param string $alias view alias
     *
     * @return string absolute view path
     */
    protected function getViewPath($alias)
    {
        $app = Yii::app();
        $context = $app->controller instanceof CController ? $app->controller : $app->getComponent('viewRenderer');
        return $context instanceof CController ? $context->getViewFile($alias) : Yii::getPathOfAlias($alias) . $context->fileExtension;
    }

    
    /**
     * Renders inernal view file
     * 
     * @param string  $viewFile view file path
     * @param array   $data     view data
     * @param boolean $return   whether required to return output content
     * 
     * @return mixed generated content
     */
    public function renderInternal($viewFile, $data, $return = false)
    {
        if (!empty($data)) {
            extract($data);
        }
        if ($return) {
            ob_start();
            ob_implicit_flush(false);
            include $viewFile;
            return ob_get_clean();
        } else {
            include $viewFile;
        }
    }

    /**
     * Constructs the emailer object
     *
     * @return void
     */
    public function __construct()
    {
        $this->_myMailer = new PHPMailer();
    }

    /**
     * PHP defined magic method __call
     *
     * @param string $method method name
     * @param array  $params method parameters
     * 
     * @return void
     */
    public function __call($method, $params)
    {
        if (is_object($this->_myMailer) && get_class($this->_myMailer) === 'PHPMailer') {
            return call_user_func_array(array($this->_myMailer, $method), $params);
        } else {
            throw new CException(Yii::t('EMailer', 'Can not call a method of a non existent object'));
        }
    }

    /**
     * PHP defined magic method __set
     *
     * @param string $name  property name
     * @param mixed  $value property value
     *
     * @return void
     * @throws CException
     */
    public function __set($name, $value)
    {
        if (is_object($this->_myMailer) && get_class($this->_myMailer) === 'PHPMailer') {
            $this->_myMailer->$name = $value;
        } else {
            throw new CException(Yii::t('EMailer', 'Can not set a property of a non existent object'));
        }
    }

    /**
     * PHP defined magic method __get
     *
     * @param string $name property name
     *
     * @return mixed property value
     * @throws CException
     */
    public function __get($name)
    {
        if (is_object($this->_myMailer) && get_class($this->_myMailer) === 'PHPMailer') {
            return $this->_myMailer->$name;
        } else {
            throw new CException(Yii::t('EMailer', 'Can not access a property of a non existent object'));
        }
    }

    /**
     * Cleanup work before serializing.
     * This is a PHP defined magic method.
     * 
     * @return array the names of instance-variables to serialize.
     */
    public function __sleep()
    {
    }

    /**
     * This method will be automatically called when unserialization happens.
     * This is a PHP defined magic method.
     *
     * @return void
     */
    public function __wakeup()
    {
    }
}